//-----------------------------------------------------------------------
// <copyright file="DefaultSubscriptionSelectorPolicyFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using Rhino.Mocks;

    using TestObjects;

    using Utility;

    using Xunit;

    public class DefaultSubscriptionSelectorPolicyFixture
    {
        [Fact]
        public void ReflectingOverObjectWithoutSubscriptionResultsInEmptyPolicy()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();

            context.ExecuteBuildUp(buildKey, null);

            ISubscriptionSelectorPolicy subscriptionSelectorPolicy =
                    context.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);

            Assert.NotNull(subscriptionSelectorPolicy);

            List<SubscriptionInfo> subscriptionInfos =
                    new List<SubscriptionInfo>(subscriptionSelectorPolicy.GetSubscriptionInfos(context));

            Assert.Equal(0, subscriptionInfos.Count);
        }

        [Fact]
        public void StrategyDoesntOverwriteAnExistingPolicy()
        {
            MockBuilderContext context = CreateContext();

            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();
            ISubscriptionSelectorPolicy policy = MockRepository.GenerateMock<ISubscriptionSelectorPolicy>();
            context.Policies.Set(policy, buildKey);

            context.ExecuteBuildUp(buildKey, null);

            ISubscriptionSelectorPolicy setPolicy =
                    context.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);
            Assert.Same(policy, setPolicy);
        }

        [Fact]
        public void ReflectingOverPublishingTypeResultsInCorrectPolicy()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventPublisher>();

            context.ExecuteBuildUp(buildKey, null);

            ISubscriptionSelectorPolicy policy = context.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);

            Assert.NotNull(policy);

            List<SubscriptionInfo> subscriptions = new List<SubscriptionInfo>(policy.GetSubscriptionInfos(context));

            Assert.Equal(0, subscriptions.Count);
        }

        [Fact]
        public void ReflectingOverSubscribingTypeResultsInCorrectPolicy()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventSubscriber>();

            context.ExecuteBuildUp(buildKey, null);

            ISubscriptionSelectorPolicy policy = context.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);

            Assert.NotNull(policy);

            List<SubscriptionInfo> subscriptions = new List<SubscriptionInfo>(policy.GetSubscriptionInfos(context));

            CollectionAssert.AreEqual(new SubscriptionInfo[] { new SubscriptionInfo("copy", typeof(OneEventSubscriber).GetMethod("OnCopy")) }, subscriptions);
        }

        [Fact]
        public void OneTypeCanPublishAndSubscribeMultipleTimes()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<ClipboardManager>();

            context.ExecuteBuildUp(buildKey, null);

            ISubscriptionSelectorPolicy policy = context.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);

            Assert.NotNull(policy);

            List<SubscriptionInfo> subscriptions = new List<SubscriptionInfo>(policy.GetSubscriptionInfos(context));

            subscriptions.Sort(
                    delegate(SubscriptionInfo a, SubscriptionInfo b)
                    {
                        return a.EventTopic.CompareTo(b.EventTopic);
                    });

            CollectionAssert.AreEqual(
                    new SubscriptionInfo[]
                        {
                                new SubscriptionInfo(
                                        "clipboard data available",
                                        typeof(ClipboardManager).GetMethod("OnClipboardDataAvailable")),
                                new SubscriptionInfo("copy", typeof(ClipboardManager).GetMethod("OnCopy")),
                        },
                    subscriptions);
        }

        private static MockBuilderContext CreateContext()
        {
            MockBuilderContext context = new MockBuilderContext();
            context.Policies.SetDefault<ISubscriptionSelectorPolicy>(new DefaultSubscriptionSelectorPolicy());

            return context;
        }
    }
}