﻿//-----------------------------------------------------------------------
// <copyright file="PublicationWireupStrategyFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using EventBroker;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using Rhino.Mocks;

    using TestObjects;

    using Utility;

    using Xunit;

    public class PublicationWireupStrategyFixture
    {
        [Fact]
        public void StrategyThrowsIfWireupIsNeededAndBrokerNotInLocator()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventPublisher>();

            IPublicationSelectorPolicy policy = MockRepository.GenerateMock<IPublicationSelectorPolicy>();
            context.Policies.SetDefault(policy);

            Assert.Throws<BuildFailedException>(
                    delegate
                    {
                        OneEventPublisher existing = new OneEventPublisher();
                        context.ExecuteBuildUp(buildKey, existing);
                    });
        }

        [Fact]
        public void NoExceptionIfExistingObjectDoesntAndNoBroker()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventPublisher>();

            IPublicationSelectorPolicy policy = MockRepository.GenerateMock<IPublicationSelectorPolicy>();
            context.Policies.SetDefault(policy);

            context.ExecuteBuildUp(buildKey, null);
            
            // No assert needed, if we got here, we're ok
        }

        [Fact]
        public void ExceptionIfNoWireupNeededAndNoBroker()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();

            IPublicationSelectorPolicy policy = MockRepository.GenerateMock<IPublicationSelectorPolicy>();
            context.Policies.SetDefault(policy);

            Assert.Throws<BuildFailedException>(
                    delegate
                    {
                        context.ExecuteBuildUp(buildKey, new object());
                    });
        }

        [Fact]
        public void StrategyProperlyWiresEvents()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();

            MockRepository mocks = new MockRepository();
            IEventBroker broker = mocks.StrictMock<IEventBroker>();
            context.Locator.Add(typeof(IEventBroker), broker);

            IPublicationSelectorPolicy policy = mocks.StrictMock<IPublicationSelectorPolicy>();
            context.Policies.SetDefault(policy);

            object existing = new object();

            using (mocks.Record())
            {
                PublicationInfo[] publicationInfos = new PublicationInfo[]
                                                         {
                                                                 new PublicationInfo(
                                                                         "cut", typeof(ClipboardManager).GetEvent("Cut")),
                                                                 new PublicationInfo(
                                                                         "copy",
                                                                         typeof(ClipboardManager).GetEvent("Copy")),
                                                                 new PublicationInfo(
                                                                         "paste",
                                                                         typeof(ClipboardManager).GetEvent("Paste")),
                                                         };

                Expect.Call(policy.GetPublicationInfos(context)).Return(publicationInfos);

                using (mocks.Unordered())
                {
                    broker.RegisterPublication("cut", existing, "Cut");
                    broker.RegisterPublication("copy", existing, "Copy");
                    broker.RegisterPublication("paste", existing, "Paste");
                }
            }

            mocks.ReplayAll();

            context.ExecuteBuildUp(buildKey, existing);

            mocks.VerifyAll();
        }

        [Fact]
        public void StrategyUnregisterPublisher()
        {
            MockBuilderContext context = CreateContext();
            
            MockRepository mocks = new MockRepository();
            IEventBroker broker = mocks.StrictMock<IEventBroker>();

            context.Locator.Add(typeof(IEventBroker), broker);

            object existing = new object();

            using (mocks.Record())
            {
                broker.UnregisterPublisher(existing);
            }

            mocks.ReplayAll();

            context.ExecuteTearDown(existing);

            mocks.VerifyAll();
        }

        private static MockBuilderContext CreateContext()
        {
            MockBuilderContext context = new MockBuilderContext();
            context.Strategies.Add(new PublicationWireupStrategy());

            return context;
        }
    }
}
