//-----------------------------------------------------------------------
// <copyright file="RegistrationSubscriptionFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;
    using Microsoft.Practices.Unity;

    using TestObjects;

    using Utility;

    using Xunit;

    public class RegistrationSubscriptionFixture : RegistrationMemberFixtureBase
    {
        [Fact]
        public void QualifyingInjectionSubscriptionCanBeConfigured()
        {
            IUnityContainer container =
                    this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ClipboardManager>(new RegistrationSubscription("OnCopy", "copy me"))
                            .Container;

            ClipboardManager existing = container.Resolve<ClipboardManager>();

            IBuildKey buildKey = NamedTypeBuildKey.Make<ClipboardManager>();
            ISubscriptionSelectorPolicy policy =
                    this.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);

            Assert.IsType<SpecifiedSubscriptionSelectorPolicy>(policy);

            MockBuilderContext context = new MockBuilderContext();
            context.ExecuteBuildUp(buildKey, existing);
            IEnumerable<SubscriptionInfo> publicationInfos = policy.GetSubscriptionInfos(context);

            Assert.Contains(new SubscriptionInfo("copy me", typeof(ClipboardManager).GetMethod("OnCopy")), publicationInfos);
            Assert.DoesNotContain(new SubscriptionInfo("copy", typeof(ClipboardManager).GetMethod("OnCopy")), publicationInfos);
            Assert.DoesNotContain(new SubscriptionInfo("clipboard data available", typeof(ClipboardManager).GetMethod("OnClipboardDataAvailable")), publicationInfos);
        }

        [Fact]
        public void CannotConfigureWrongMethod()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ClipboardManager>(new RegistrationSubscription("Fly", "fly"));
                    });
        }

        [Fact]
        public void CannotConfigureMethodWithWrongParams()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithWrongParams>(new RegistrationSubscription(
                                "MethodWithOneParam",
                                "one",
                                typeof(EventArgs)));
                    });

            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithWrongParams>(new RegistrationSubscription(
                                "MethodWithThreeParams",
                                "three",
                                typeof(object),
                                typeof(EventArgs),
                                typeof(object)));
                    });
        }

        [Fact]
        public void CannotConfigureMethodWithReturn()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithReturn>(new RegistrationSubscription(
                                "MethodWithReturn",
                                "return",
                                typeof(object),
                                typeof(EventArgs)));
                    });
        }

        [Fact]
        public void CannotConfigureStaticMethod()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<StaticObject>(new RegistrationSubscription(
                                "StaticMethod",
                                "static",
                                typeof(object),
                                typeof(EventArgs)));
                    });
        }

        [Fact]
        public void CannotConfigureMethodWithOutParams()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithOutParams>(new RegistrationSubscription(
                                "MethodWithOutParams",
                                "out"));
                    });
        }

        [Fact]
        public void CannotConfigureMethodWithRefParams()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithRefParams>(new RegistrationSubscription(
                                "MethodWithRefParams",
                                "ref"));
                    });
        }

        [Fact]
        public void CannotConfigureGenericMethod()
        {
            this.CreateContainer()
                .Configure<InjectedMembers>()
                .ConfigureInjectionFor(typeof(ObjectWithGenericMethod<>), new RegistrationSubscription("NoneGenericMethod", "none generic"));

            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithGenericMethod<object>>(new RegistrationSubscription("GenericMethod", "generic"));
                    });
        }

        private class ObjectWithGenericMethod<TSender>
            where TSender : class
        {
            public void GenericMethod<T>(TSender sender, T e)
                where T : EventArgs
            {
            }

            public void NoneGenericMethod(TSender sender, EventArgs e)
            {
            }
        }
    }
}