//-----------------------------------------------------------------------
// <copyright file="SubscriptionWireupStrategyFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;

    using EventBroker;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using Rhino.Mocks;

    using TestObjects;

    using Utility;

    using Xunit;

    public class SubscriptionWireupStrategyFixture
    {
        [Fact]
        public void StrategyThrowsIfWireupIsNeededAndBrokerNotInLocator()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventPublisher>();

            ISubscriptionSelectorPolicy policy = MockRepository.GenerateMock<ISubscriptionSelectorPolicy>();
            context.Policies.SetDefault(policy);

            Assert.Throws<BuildFailedException>(
                    delegate
                    {
                        OneEventPublisher existing = new OneEventPublisher();
                        context.ExecuteBuildUp(buildKey, existing);
                    });
        }

        [Fact]
        public void NoExceptionIfExistingObjectDoesntAndNoBroker()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventPublisher>();

            ISubscriptionSelectorPolicy policy = MockRepository.GenerateMock<ISubscriptionSelectorPolicy>();
            context.Policies.SetDefault(policy);

            context.ExecuteBuildUp(buildKey, null);

            // No assert needed, if we got here, we're ok
        }

        [Fact]
        public void ExceptionIfNoWireupNeededAndNoBroker()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();

            ISubscriptionSelectorPolicy policy = MockRepository.GenerateMock<ISubscriptionSelectorPolicy>();
            context.Policies.SetDefault(policy);

            Assert.Throws<BuildFailedException>(
                    delegate
                    {
                        context.ExecuteBuildUp(buildKey, new object());
                    });
        }

        [Fact]
        public void StrategyProperlyWiresEvents()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();

            MockRepository mocks = new MockRepository();
            IEventBroker broker = mocks.StrictMock<IEventBroker>();
            context.Locator.Add(typeof(IEventBroker), broker);

            ISubscriptionSelectorPolicy policy = mocks.StrictMock<ISubscriptionSelectorPolicy>();
            context.Policies.SetDefault(policy);

            object existing = new object();

            using (mocks.Record())
            {
                SubscriptionInfo[] subscriptionInfos = new SubscriptionInfo[]
                                                           {
                                                                   new SubscriptionInfo(
                                                                           "copy",
                                                                           typeof(ClipboardManager).GetMethod("OnCopy")),
                                                                   new SubscriptionInfo(
                                                                           "clipboard data available",
                                                                           typeof(ClipboardManager).GetMethod("OnClipboardDataAvailable")),
                                                           };

                Expect.Call(policy.GetSubscriptionInfos(context)).Return(subscriptionInfos);

                using (mocks.Unordered())
                {
                    foreach (SubscriptionInfo subscriptionInfo in subscriptionInfos)
                    {
                        List<Type> list = new List<Type>();
                        foreach (ParameterInfo parameterInfo in subscriptionInfo.Method.GetParameters())
                        {
                            list.Add(parameterInfo.ParameterType);
                        }

                        broker.RegisterSubscription(subscriptionInfo.EventTopic, existing, subscriptionInfo.Method.Name, list.ToArray());
                    }
                }
            }

            mocks.ReplayAll();

            context.ExecuteBuildUp(buildKey, existing);

            mocks.VerifyAll();
        }

        [Fact]
        public void StrategyUnregisterSubscriber()
        {
            MockBuilderContext context = CreateContext();

            MockRepository mocks = new MockRepository();
            IEventBroker broker = mocks.StrictMock<IEventBroker>();

            context.Locator.Add(typeof(IEventBroker), broker);

            object existing = new object();

            using (mocks.Record())
            {
                broker.UnregisterSubscriber(existing);
            }

            mocks.ReplayAll();

            context.ExecuteTearDown(existing);

            mocks.VerifyAll();
        }

        private static MockBuilderContext CreateContext()
        {
            MockBuilderContext context = new MockBuilderContext();
            context.Strategies.Add(new SubscriptionWireupStrategy());

            return context;
        }
    }
}