using System;
using System.Collections.Generic;
using System.Reflection;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class GenericClassesInterfaceInterceptorTest
    {
        [Fact]
        public void GenericMethod()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(IGenericSpy<>).GetMethod("GenericMethod");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            IGenericSpy<int> result = WrapAndCreateType<IGenericSpy<int>, GenericSpy<int>>(dictionary);
            result.GenericMethod(46, "2");

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data 46 and 2", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        [Fact]
        public void NonGenericMethod()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(IGenericSpy<>).GetMethod("MethodWhichTakesGenericData");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            IGenericSpy<int> result = WrapAndCreateType<IGenericSpy<int>, GenericSpy<int>>(dictionary);
            result.MethodWhichTakesGenericData(24);

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data 24", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        [Fact]
        public void ReturnsDataOfGenericType()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(IGenericSpy<>).GetMethod("GenericReturn");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            IGenericSpy<int> result = WrapAndCreateType<IGenericSpy<int>, GenericSpy<int>>(dictionary);
            int value = result.GenericReturn(256.9);

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data 256.9", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
            Assert.Equal(default(int), value);
        }

        [Fact]
        public void WhereClauseOnInterface()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(IGenericSpyWithWhereClause<>).GetMethod("Method");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            IGenericSpyWithWhereClause<Foo> result = WrapAndCreateType<IGenericSpyWithWhereClause<Foo>, GenericSpyWithWhereClause<Foo>>(dictionary);
            result.Method(new Foo());

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data " + typeof(Foo).FullName, Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        static TInterface WrapAndCreateType<TInterface, TConcrete>(IEnumerable<KeyValuePair<MethodBase, List<IInterceptionHandler>>> handlers)
            where TConcrete : TInterface
        {
            Type wrappedType = InterfaceInterceptor.WrapInterface(typeof(TInterface));
            ILEmitProxy proxy = new ILEmitProxy(handlers);
            object target = Activator.CreateInstance(typeof(TConcrete));
            List<object> wrappedCtorArgs = new List<object> { proxy, target };
            return (TInterface)Activator.CreateInstance(wrappedType, wrappedCtorArgs.ToArray());
        }

        public class Foo : IFoo {}

        public class GenericSpy<T> : IGenericSpy<T>
        {
            public void GenericMethod<T1>(T data,
                                          T1 data1)
            {
                Recorder.Records.Add("In method with data " + data + " and " + data1);
            }

            public T GenericReturn<T1>(T1 data)
            {
                Recorder.Records.Add("In method with data " + data);
                return default(T);
            }

            public void MethodWhichTakesGenericData(T data)
            {
                Recorder.Records.Add("In method with data " + data);
            }
        }

        public class GenericSpyWithWhereClause<T> : IGenericSpyWithWhereClause<T>
            where T : class, IFoo
        {
            public void Method(T data)
            {
                Recorder.Records.Add("In method with data " + data);
            }
        }

        public interface IFoo {}

        public interface IGenericSpy<T>
        {
            void GenericMethod<T1>(T data,
                                   T1 data1);

            T GenericReturn<T1>(T1 data);
            void MethodWhichTakesGenericData(T data);
        }

        public interface IGenericSpyWithWhereClause<T>
            where T : class, IFoo
        {
            void Method(T data);
        }
    }
}