using System;
using System.Collections.Generic;
using System.Reflection;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class ReturnValuesVirtualInterceptorTest
    {
        [Fact]
        public void Exception()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyReturn).GetMethod("Exception");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyReturn result = WrapAndCreateType<SpyReturn>(dictionary, 42);
            Assert.Throws<ArgumentException>(result.Exception);
        }

        [Fact]
        public void NoReturnValue()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyReturn).GetMethod("NoReturnValue");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyReturn result = WrapAndCreateType<SpyReturn>(dictionary, 42);
            result.NoReturnValue();

            Assert.Equal(42, result.ConstructorValue);
            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In Method", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        [Fact]
        public void ReturnsClassType()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyReturn).GetMethod("ReturnsClassType");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyReturn result = WrapAndCreateType<SpyReturn>(dictionary, 42);
            object retValue = result.ReturnsClassType();

            Assert.Same(SpyReturn.ObjectReturn, retValue);
        }

        [Fact]
        public void ReturnsValueType()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyReturn).GetMethod("ReturnsValueType");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyReturn result = WrapAndCreateType<SpyReturn>(dictionary, 42);
            int retValue = result.ReturnsValueType();

            Assert.Equal(SpyReturn.ValueReturn, retValue);
        }

        static T WrapAndCreateType<T>(IEnumerable<KeyValuePair<MethodBase, List<IInterceptionHandler>>> handlers,
                                      params object[] ctorArgs)
        {
            Type wrappedType = VirtualInterceptor.WrapClass(typeof(T));
            ILEmitProxy proxy = new ILEmitProxy(handlers);
            List<object> wrappedCtorArgs = new List<object> { proxy };
            wrappedCtorArgs.AddRange(ctorArgs);
            return (T)Activator.CreateInstance(wrappedType, wrappedCtorArgs.ToArray());
        }

        public class SpyReturn
        {
            public const int ValueReturn = 42;
            public static object ObjectReturn = new object();
            public readonly int ConstructorValue;

            public SpyReturn(int x)
            {
                ConstructorValue = x;
            }

            public virtual void Exception()
            {
                throw new ArgumentException();
            }

            public virtual void NoReturnValue()
            {
                Recorder.Records.Add("In Method");
            }

            public virtual object ReturnsClassType()
            {
                return ObjectReturn;
            }

            public virtual int ReturnsValueType()
            {
                return ValueReturn;
            }
        }
    }
}