using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;

namespace Unity.Extensions
{
    /// <summary>
    /// Represents a paramenter collection.
    /// </summary>
    /// <remarks>
    /// A collection of the arguments and the <see cref="ParameterInfo"/> objects to 
    /// describe the arguments of a method / constructor.
    /// </remarks>
    public class ParameterCollection : IParameterCollection
    {
        readonly List<ArgumentInfo> argumentInfo;
        readonly object[] arguments;

        /// <summary>
        /// Initializes a new instance of <see cref="ParameterCollection"/> class with an array of the argument values and 
        /// an array of the <see cref="ParameterInfo"/> describing the values.
        /// </summary>
        /// <param name="arguments">An array of the argument values.</param>
        /// <param name="argumentInfo"></param>
        public ParameterCollection(object[] arguments,
                                   ParameterInfo[] argumentInfo)
            : this(arguments, argumentInfo, delegate
                                            {
                                                return true;
                                            }) {}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="arguments"></param>
        /// <param name="argumentInfo"></param>
        /// <param name="isArgumentPartOfCollection"></param>
        protected ParameterCollection(object[] arguments,
                                      ParameterInfo[] argumentInfo,
                                      Predicate<ParameterInfo> isArgumentPartOfCollection)
        {
            this.arguments = arguments;
            this.argumentInfo = new List<ArgumentInfo>();

            for (int idx = 0; idx < argumentInfo.Length; ++idx)
                if (isArgumentPartOfCollection(argumentInfo[idx]))
                    this.argumentInfo.Add(new ArgumentInfo(idx, argumentInfo[idx]));
        }

        /// <summary>
        /// Gets or sets the parameter value at the specified index.
        /// </summary>
        /// <param name="index"></param>
        /// <returns>The parameter value at the specified index.</returns>
        public object this[int index]
        {
            get { return arguments[argumentInfo[index].Index]; }
            set { arguments[argumentInfo[index].Index] = value; }
        }

        public object this[string paramName]
        {
            get { return arguments[IndexForParameterName(paramName)]; }
            set { arguments[IndexForParameterName(paramName)] = value; }
        }

        public IEnumerator GetEnumerator()
        {
            for (int i = 0; i < argumentInfo.Count; ++i)
                yield return arguments[argumentInfo[i].Index];
        }

        int IndexForParameterName(string paramName)
        {
            for (int i = 0; i < argumentInfo.Count; ++i)
                if (argumentInfo[i].Name == paramName)
                    return argumentInfo[i].Index;

            throw new ArgumentException("Invalid parameter Name", "paramName");
        }

        struct ArgumentInfo
        {
            public readonly int Index;
            public readonly string Name;

            public ArgumentInfo(int index,
                                ParameterInfo parameterInfo)
            {
                Index = index;
                Name = parameterInfo.Name;
            }
        }
    }
}